package com.lantone.qc.kernel.catalogue.beilun.behospitalized;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.kernel.util.CatalogueUtil;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.entity.Diag;
import com.lantone.qc.pub.model.label.PastLabel;
import com.lantone.qc.pub.util.ListUtil;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description: 疾病名未标引号
 * @author: rengb
 * @time: 2020/3/10 14:02
 */
@Component
public class BEH0032 extends QCCatalogue {

    @Override
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        status.set("0");
        if (inputInfo.getBeHospitalizedDoc() == null) {
            return;
        }
        PastLabel pastLabel = inputInfo.getBeHospitalizedDoc().getPastLabel();
        if (pastLabel == null) {
            return;
        }
        List<Diag> diags = pastLabel.getDiags();
        if (ListUtil.isEmpty(diags)) {
            return;
        }
        List<String> wtDiagNames = CatalogueUtil.noInQuotes(
                diags.stream().filter(diag -> diag.getNegative() == null).map(i -> i.getHospitalDiagName()).filter(i -> StringUtil.isNotEmpty(i)).distinct().collect(Collectors.toList()),
                pastLabel.getText().replace("“", "\"").replace("”", "\"")
        );
        String pastText = pastLabel.getText();
        List<String> wtDiagList = new ArrayList<>();
        if (ListUtil.isNotEmpty(wtDiagNames)) {
            for (String wtDiagName : wtDiagNames) {
                int index = pastText.indexOf(wtDiagName);
                if (index - 1 > 0) {
                    String markText = pastText.substring(index - 1, index);
                    if (markText.contains("\"") || markText.contains("“")) {
                        continue;
                    }
                }
                if (index + wtDiagName.length() + 1 < pastText.length() && index > 0) {
                    String markText = pastText.substring(index + wtDiagName.length(), index + wtDiagName.length() + 1);
                    if (markText.contains("\"") || markText.contains("”")) {
                        continue;
                    }
                }
                wtDiagList.add(wtDiagName);
            }
        }

        if (ListUtil.isNotEmpty(wtDiagList)) {
            status.set("-1");
            for (String wtDiag : wtDiagList) {
                info.set(info.get() + wtDiag + " ");
            }
        }
    }

}