package com.lantone.qc.kernel.catalogue.yiwu.behospitalized;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.BeHospitalizedDoc;
import com.lantone.qc.pub.model.entity.Vaccinate;
import com.lantone.qc.pub.model.label.PastLabel;
import com.lantone.qc.pub.model.label.PersonalLabel;
import com.lantone.qc.pub.util.ListUtil;
import com.lantone.qc.pub.util.StringUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @Description: 预防接种史未填写
 * @author: rengb
 * @time: 2020/3/10 14:02
 */
@Component
public class BEH0029 extends QCCatalogue {

    @Override
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        if (inputInfo.getBeHospitalizedDoc() == null) {
            status.set("0");
            return;
        }
        BeHospitalizedDoc beHospitalizedDoc = inputInfo.getBeHospitalizedDoc();
        //台州结构化
        Map<String, String> structureMap = beHospitalizedDoc.getStructureMap();
        if (StringUtils.isNotEmpty(structureMap.get("预防接种史"))) {
            status.set("0");
            return;
        }
        String past = structureMap.get("既往史");
        if (StringUtil.isNotBlank(past) && past.contains("预防接种")) {
            status.set("0");
            return;
        }
        PastLabel pastLabel = beHospitalizedDoc.getPastLabel();
        //个人史
        PersonalLabel personalLabel = beHospitalizedDoc.getPersonalLabel();
        /* 如果既往史为空或者既往史文本为空，则不报错 */
        if ((pastLabel == null || StringUtil.isBlank(pastLabel.getText())) &&
                (personalLabel == null || StringUtil.isBlank(personalLabel.getText()))) {
            status.set("0");
            return;
        }
        if (pastLabel != null && pastLabel.getText() != null) {
            String pastText = pastLabel.getText();
            if (pastText.contains("详见原病历") || pastText.contains("预防接种")) {
                status.set("0");
                return;
            }
            List<Vaccinate> vaccinates = pastLabel.getVaccinates();
            if (ListUtil.isNotEmpty(vaccinates)) {
                if (vaccinates.stream().map(i -> i.getName()).filter(i -> StringUtil.isNotBlank(i)).count() > 0) {
                    status.set("0");
                }
            }
        }
        if (personalLabel != null && personalLabel.getText() != null) {
            String personText = personalLabel.getText();
            if (StringUtil.isNotBlank(personText) && (personText.contains("详见原病历") || personText.contains("预防接种"))) {
                status.set("0");
                return;
            }
        }
        //直接map拿既往史、个人史
        if (structureMap.get("既往史") != null) {
            String pastText = structureMap.get("既往史");
            if (pastText.contains("详见原病历") || pastText.contains("预防接种")) {
                status.set("0");
                return;
            }
        }
        if (structureMap.get("个人史") != null) {
            String personText = structureMap.get("个人史");
            if (personText.contains("详见原病历") || personText.contains("预防接种")) {
                status.set("0");
                return;
            }
        }
    }

}