package com.lantone.qc.kernel.catalogue.yiwu.behospitalized;

import com.google.common.collect.Lists;
import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.BeHospitalizedDoc;
import com.lantone.qc.pub.model.entity.General;
import com.lantone.qc.pub.model.entity.GeneralDesc;
import com.lantone.qc.pub.model.label.PresentLabel;
import com.lantone.qc.pub.util.ListUtil;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.stereotype.Component;

import java.util.List;


/**
 * @ClassName : BEH0445
 * @Description :  现病史缺少发病以来睡眠描述
 * @Author : 楼辉荣
 * @Date: 2020-03-06 17:28
 */
@Component
public class BEH0445 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        BeHospitalizedDoc beHospitalizedDoc = inputInfo.getBeHospitalizedDoc();
        if (beHospitalizedDoc == null) {
            status.set("0");
            return;
        }
        PresentLabel presentLabel = beHospitalizedDoc.getPresentLabel();
        if (presentLabel == null) {
            status.set("0");
            return;
        }
        String presentText = presentLabel.getText();
        if (StringUtil.isNotBlank(presentText)) {
            List<String> words = Lists.newArrayList("睡眠", "未眠", "未睡", "未入眠", "入睡", "纳眠", "夜眠", "镇静", "未开奶","吃奶","已开奶");
            for (String word : words) {
                if (presentText.contains(word)) {
                    status.set("0");
                    return;
                }
            }
        }
        List<GeneralDesc> generals = presentLabel.getGenerals();
        List<General> gens = presentLabel.getGens();
        if (ListUtil.isEmpty(generals) && ListUtil.isEmpty(gens)) {
            status.set("0");
            return;
        }
        if (ListUtil.isNotEmpty(generals)) {
            for (GeneralDesc general : generals) {
                if (general.getName().contains("睡眠") || general.getName().contains("纳眠") || general.getName().contains("入睡")) {
                    status.set("0");
                    break;
                }
            }
        }

        if (ListUtil.isNotEmpty(gens)) {
            for (General general : gens) {
                if (general.getName().contains("睡眠") || general.getName().contains("纳眠") || general.getName().contains("入睡")) {
                    status.set("0");
                    break;
                }
            }
        }
    }
}