package com.lantone.qc.kernel.catalogue.yiwu.clinicalblood;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.ClinicalBloodDoc;
import com.lantone.qc.pub.util.ListUtil;
import com.lantone.qc.pub.util.StringUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @ClassName : CLI0306
 * @Description : 输血记录中无输血指征
 * CRF缺少足够标注数据，使用规则判断输血原因是否有数字
 * @Author : 楼辉荣
 * @Date: 2020-03-06 17:28
 */
@Component
public class CLI0306 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        status.set("0");
        List<ClinicalBloodDoc> clinicalBloodDocs = inputInfo.getClinicalBloodDocs();
        if (ListUtil.isEmpty(clinicalBloodDocs)) {
            return;
        }
        if (clinicalBloodDocs != null && clinicalBloodDocs.size() > 0) {
            for (ClinicalBloodDoc cliB : clinicalBloodDocs) {
                Map<String, String> cliBStructureMap = cliB.getStructureMap();
                //台州
                if (StringUtils.isNotEmpty(cliBStructureMap.get("输血检验指标"))) {
                    continue;
                }
                String infusionReason = cliBStructureMap.get("输注原因");
                if (StringUtils.isNotEmpty(infusionReason)) {
                    continue;
                }
                String title = cliBStructureMap.get("病程标题");
                if (StringUtil.isNotBlank(title) && title.contains("蛋白")){
                    continue;
                }
                String record = cliBStructureMap.get("病情记录");
                if (!record.contains("贫血")&&!record.contains("低蛋白")) {
                    status.set("-1");
                    break;
                }
            }
        } else {
            status.set("0");
        }

    }
}