package com.lantone.qc.kernel.catalogue.yiwu.preoperativediscussion;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.operation.OperationDoc;
import com.lantone.qc.pub.model.doc.operation.PreoperativeDiscussionDoc;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.util.List;
import java.util.Map;

/**
 * @Description: 术前讨论记录无手术指征
 * @author: rengb
 * @time: 2020/3/23 15:09
 */
@Component
public class PRE0329 extends QCCatalogue {

    @Override
    protected void start(InputInfo inputInfo, OutputInfo outputInfo) throws ParseException {
        status.set("0");
        boolean isOperativePatient = true;//是手术患者(暂时默认是)
        List<OperationDoc> operationDocs = inputInfo.getOperationDocs();
        if (operationDocs == null || operationDocs.size() == 0) {
            return;
        }
        for (OperationDoc operationDoc : operationDocs) {
            PreoperativeDiscussionDoc doc = operationDoc.getPreoperativeDiscussionDoc();
            if (doc == null) {
                continue;
            }
            Map<String, String> map = doc.getStructureMap();
            String originText = map.get("原始文本");
            if (StringUtil.isNotBlank(originText) && originText.contains("手术指征")) {
                continue;
            }
            if (map.containsKey("手术指征与禁忌") && StringUtil.isEmpty(map.get("手术指征与禁忌"))) {
                status.set("-1");
            }
        }

    }

}